  ' Battery Management Program for Caravan 12V System          '
  ' Written by Doug Pankhurst May 2017, based on ideas from    '
  ' Tom Pankhurst and various Back Shed Forum members.         '                                    '
  ' Written for MMBasic V5.4.05  , copyright Geoff Graham 2012 '
  ' This program is free for anyone to use or modify as they   '
  '  choose however, if you wish to re-distribute, please      '
  '  acknowledge authors and contributors above.               '
  ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  ' File Version cvbmbe32.bas  25 July 2017
  ' last known good version 2.44
  ' make all display code subroutines
  ' Upgrade from CGCMM2 platform to matherp uM+ Backpack
  ' with 7" SSD1963 display
  ' - still looks for uM170 front end to provide data
  ' This version does NOT use any GUI functions.
  ' For a detailed explanation of the program, it's
  ' construction and rationale, please read the
  ' CSM3READ.TXT file.
              
  ' Main Entry to Program
EntryPoint:
  SetTick 0,0        ' Disable timer interrupt
  Option explicit
  ' Global Variables - initialised to default values
  Dim F_Ver = 3.2    ' File Version cvbmbe32.bas  8 February 2017
  ' Voltage and current of caravan battery
  Dim BVolts = 12.8  '  Battery Voltage
  Dim BVArray(29)    ' Battary Volts 30 element array - 1 each 2 secs
  Dim AverageBV=12.8 ' Average battery voltage over last 60 seconds
  Dim PreviousABV=12.8 '
  Dim OldABV = 13    ' so we can calculate ABV correctly
  Dim DeltaBV = 0.1  ' Battery voltage variation over 60 seconds. The
  ' difference between the first and the last used to
  ' calculate the "Time to 30% discharge point"
  Dim Time2Discharge = 20 ' Time to 30% discharge in minutes
  Dim BatCharge = 100' Battery state of charge as a percentage where
  ' 12.75V or higher is 100%, down to 12.05V being 35% - danger level
  ' Voltage and current from Solar Panels
  Dim SVolts = 20    ' Solar Array voltage
  Dim SVArray(29)    ' Solar Volts 30 element array - 1 each two secs
  Dim AverageSV = 21 ' Average Solar Panel volts over last minute
  Dim SAmps = 0 ' current from Solar Array into D250
  Dim SAArray(29)    ' solar panel current array - 1 each two secs
  Dim AverageSC = 0  ' average solar panerl current last minute
  ' Voltage and current from towing vehicle
  Dim CVolts = 13.8  ' Vehicle battery volts
  Dim CVArray(29)    ' Car volts 30 element array  - 1 each two secs
  Dim AverageCV = 13 ' Average vehicle volts ove rlast 30 60 secs
  Dim CAmps = 0 ' current from vehicle into D250
  Dim CAArray(29)    ' car current array  - 1 each two secs
  Dim AverageCC = 0  ' average car current last minute
  ' Input (charging), output (load) and battery charge/discharge currents.
  ' Battery charge/discharge is calculated from input minus load currents.
  ' - instantaneous values, minute averaged value currents
  ' - daily accumulating input, output and battery amp hour useage
  ' - last 4 days battery charge/discharge currents and rolling total
  Dim ACAmps = 0 ' Charging current from XPS25000 240VAC charger
  Dim DCAmps = 0 ' Charging current from D250 Solar Charger
  Dim ACAArray(29)   ' XPS25000 240VAC minute current array - 1 each two secs
  Dim DCAArray(29)   ' D250 minute current array  - 1 each 2 secs
  Dim AverageACC = 0 ' average D250 output current last minute
  Dim AverageDCC = 0 ' average X25000 output current last minute
  ' Input current -  a calculated sum of AC Charger and DC charger currents
  Dim IAmps = 18  ' Charging current into battery/load
  Dim AverageIC = 18 ' Average input current
  Dim LAmps = 12 ' Load current - supplied from batteries/charging systems
  Dim LAArray(29)    ' Load Current 60 element array
  Dim AverageLC = 12 ' Average load current
  ' Battery current - a calculated value; if positive, = a charging current to
  ' the battery; if negative, equals a discharge current from the battery.
  Dim BAmps = 6     ' Input charging current minus load current
  Dim AverageBC = 6      ' Average charge/discharge current
  ' daily accumulating values
  Dim InputWH = 0        ' Input Watt Hours
  Dim LoadWH = 0         ' Load Watt Hours
  ' previous days values
  Dim PrevIWH = 12 ' yesterdays Input Current ahs
  Dim PrevLWH = 10 ' yesterdays Load Current ahs
  Dim BatWH = 0   ' Bat charge (+ value) or discharge (- value) in Whs
  Dim PrevBWH = 2 ' yesterdays Bat Current Wh
  Dim Day2BWH = 20      ' last 4 days battery Watt hour usage
  Dim Day3BWH = 30
  Dim Day4BWH = 40
  Dim TotalBWH = 90     ' rolling 4 day watt hour total usage
  ' Flags  
  Dim NewDataFlag = 0   ' Two second flag - set every
  ' 2 seconds by the interrupt routine when new voltage
  ' and current values are collected.
  Dim MinFlag = 0 ' Minute flag - set once every min at 00 seconds
  ' by sec interrupt process and cleared by min processing code.
  Dim EODFlag = 0        ' End of Day Flag - set at 6AM
  dim hrs = 6
  dim mins = 0
  dim secs = 0
  Dim DataVals$ length 64 '
  Dim DataValid = 1      ' 0 = bad data, anything else is good data
  ' misc variables
  Dim Ax                 'loop counter
  Dim S_Bright=25  ' default to 25%
  ' some variables to simulate a 6Amp discharge rate
  dim cf1 = 14     ' AverageBC  - offset of x from zero
  DIM cf2 = 0.15   ' shape of curve
  DIM cf3 = 11.3   ' 0% charge point
  dim tbv = 13
  dim twosecled = 1 ' LED flashes on 2 sec interrupt
  '''''''''''''''''
  ' Carry out initialisation routines here - enable interrupt last

Entry:
  BackLight S_Bright
  Colour RGB(White),RGB(Black)   'fg and bg
  CLS
  RTC GETTIME        ' date and time from DS3231
  ' Data values come in from front end via COM1
  ' Note: Front end action could be included in this program if the
  '   system is close enough to read values directly
  Open "COM1:9600,128,GetVals, 1" As #1

  ' Display all static data
  DisplayStaticText
  DisplayStaticRunGraph
  DisplayStaticBatGraph

  ' set up any GUI controls
  GUI AREA #10,4,200,400,300 ' graph area is touch area #10
  GUI interrupt PenDown      ' subroutine to handle touch interrupt
  gui led #11,"",770,30,10,RGB(green) ' flash every 2 secs
  
  ' Fill the the minute average arrays
  ' just so initial values make sense
  For Ax=0 To 29
    BVArray(Ax) = 12.8
    SVArray(Ax) = 19
    CVArray(Ax) = 13
    SAArray(Ax) = 10
    CAArray(Ax) = 6
    ACAArray(Ax) = 0
    DCAArray(Ax) = 16
    LAArray(Ax) = 12
  Next Ax

  ' Wait until 00 seconds to start for clean entry.
  Do While Val(Mid$(Time$,7,2)) < 58
  Loop

  ' Enable timer tick interrupts to start data collection at 00 seconds
  SetTick 2000,RequestData  ' Call every other second, send ACK to front end
  ' that initiates a data dump into COM2 from the front end.
  '
  ' end of initialisation routines
  ''''''''''''''''''''

Main:
  Do    'Main program loop, for every 2 sec interrupt, get the data, do
    '  2 second, minute and end of day processing as appropriate
    Do While NewDataFlag = 0   ' wait until data request received
    ' from back end then generate fresh data and send to back end
    Pause 200
    Loop
    TwoSecProcess
    If MinFlag = 1 Then
      MinProcess
    EndIf
    If EODFlag = 1 Then
      EODProcess
    EndIf
  Loop
  ' end of main process loop
    
Sub TwoSecProcess
' entered with the following instantaneous values from front end
' battery volts, solar volts, solar current, vehicle current
' 240VAC charger current, solar/car charger current
' total input current (solar/car + 240VAC), load current
' battery current
  Local Ax2  'array counter
  ' 2 second processing code - every interrupt sets 2 sec flag
  ' display time
    Font #3,1
    Text 520,180,Time$,LT,2,2,RGB(green)
    Font #1,1
    Colour RGB(white)
    ' display date green or red for good or bad data from fe
  If DataValid = 0 Then  ' bad data or CRC from front end
    Text 1,1,Date$,L,1,1,RGB(red)
  Else ' must be good data
    Text 1,1,Date$,L,1,1,RGB(green)
  EndIf
  ' Push most recent values into arrays for minute averaging
  ' then set up for minute averaging
  For Ax2 = 29 To 1 Step -1  ' shuffle everyone down one spot
    BVArray(Ax2) = BVArray(Ax2-1)
    SVArray(Ax2) = SVArray(Ax2-1)
    CVArray(Ax2) = CVArray(Ax2-1)
    SAArray(Ax2) = SAArray(Ax2-1)
    CAArray(Ax2) = CAArray(Ax2-1)
    ACAArray(Ax2) = ACAArray(Ax2-1)
    DCAArray(Ax2) = DCAArray(Ax2-1)
    LAArray(Ax2) = LAArray(Ax2-1)
  Next Ax2
  BVArray(0) = BVolts ' then plug in new values to first spot
  SVArray(0) = SVolts
  CVArray(0) = CVolts
  SAArray(0) = SAmps
  CAArray(0) = CAmps
  ACAArray(0) = ACAmps
  DCAArray(0) = DCAmps
  LAArray(0) = LAmps
  ' Battery Current is IAmps-LAmps - if positive
  '  it is a charging current into battery, if negative, it is a
  '  discharging current out of battery
  IAmps = ACAmps + DCAmps  ' from both mains and solar/car
  BAmps = IAmps-LAmps

  ' Convert both input, output & battery instantaneous currents into Watt Hours
  InputWH = InputWH + ((IAmps/1800)*BVolts)
  LoadWH = LoadWH + ((LAmps/1800)*BVolts)
  BatWH = InputWH-LoadWH
  ' Note: All AH accumulating totals are plugged into last day values
  ' then reset to zero at 6AM - now update terminal display with all values
  ' Fill 2 second values
  Colour RGB(white)
  Text 2,52,Str$(AverageBV,2,2)+"V",L,1,4,RGB(white)
  Text 330,54,Str$(AverageBV,2,2)+"V",L,1,1,RGB(white)
  Text 330,67,Str$(AverageSV,2,1)+"V",L,1,1,RGB(cyan)
  Text 330,80,Str$(AverageCV,2,1)+"V",L,1,1,RGB(200,200,200)
  Text 330,93,Str$(AverageIC,2,1)+"A",L,1,1,RGB(yellow)
  Text 330,106,Str$(AverageLC,2,1)+"A",L,1,1,RGB(red)
  If (Sgn(AverageBC) = +1) Then
    Text 330,119,Str$(AverageBC,2,1)+"A",L,1,1,RGB(green)
    Text 480,420,Str$(BAmps,2,1)+"A",L,1,2,RGB(green)
  Else
    Text 330,119,Str$(AverageBC,2,1)+"A",L,1,1,RGB(RED)
    Text 480,420,Str$(BAmps,2,1)+"A",L,1,2,RGB(red)
  EndIf
  Text 480,243,Str$(BVolts,2,2)+"V",L,1,2,RGB(white)
  Text 650,243,Str$(CVolts,2,2)+"V",L,1,2,RGB(200,200,200)
  Text 480,283,Str$(SVolts,2,1)+"V",L,1,2,RGB(cyan)
  Text 650,283,Str$(SAmps,2,1)+"A",L,1,2,RGB(cyan)
  Text 480,323,Str$(IAmps,2,1)+"A",L,1,2,RGB(yellow)
  Text 650,323,Str$(CAmps,2,1)+"A",L,1,2,RGB(180,180,180)
  Text 480,363,Str$(LAmps,2,1)+"A",L,1,2,RGB(red)
  Text 650,363,Str$(ACAmps,2,1)+"A",L,1,2,RGB(brown)
  Text 650,420,Str$(DCAmps,2,1)+"A",L,1,2,RGB(brown)
  Text 122,171,Str$(InputWH,3,0)+"WHrs",L,1,1,RGB(white)
  Text 122,184,Str$(LoadWH,3,0)+"WHrs",L,1,1,RGB(white)
  If (Sgn(BatWH) = +1) Then
    Text 122,197,Str$(BatWH,3,0)+"WHrs",L,1,1,RGB(green)
  Else
    Text 122,197,Str$(BatWH,3,0)+"WHrs",L,1,1,RGB(red)
  EndIf
  Text 330,171,Str$(PrevIWH,3,0)+"WHrs"
  Text 330,184,Str$(PrevLWH,3,0)+"WHrs"
  If (Sgn(PrevBWH) = +1) Then
    Text 330,197,Str$(PrevBWH,3,0)+"WHrs",L,1,1,RGB(green)
  Else
    Text 330,197,Str$(PrevBWH,3,0)+"WHrs",L,1,1,RGB(red)
  EndIf
  If (Sgn(Day2BWH) = +1) Then
    Colour RGB(green)
    Text 428,54,Str$(Day2BWH,3,0)+" WHrs",L,1,1,RGB(green)
  Else
    Text 428,54,Str$(Day2BWH,3,0)+" WHrs",L,1,1,RGB(red)
  EndIf
  If (Sgn(Day3BWH) = +1) Then
    Text 428,67,Str$(Day3BWH,3,0)+" WHrs",L,1,1,RGB(green)
  Else
    Text 428,67,Str$(Day3BWH,3,0)+" WHrs",L,1,1,RGB(red)
  EndIf
  If (Sgn(Day4BWH) = +1) Then
    Text 428,80,Str$(Day4BWH,3,0)+" WHrs",L,1,1,RGB(green)
  Else
    Text 428,80,Str$(Day4BWH,3,0)+" WHrs",L,1,1,RGB(red)
  EndIf
  If (Sgn(TotalBWH) = +1) Then
    Text 428,106,Str$(TotalBWH,4,0)+" WHrs",L,1,1,RGB(green)
  Else
    Text 428,106,Str$(TotalBWH,4,0)+" WHrs",L,1,1,RGB(red)
  EndIf
  Colour RGB(white)
  NewDataFlag = 0      ' clear new data flag ready for next interrupt
  '
  ' End of two second processing code
End Sub

'''''''''''''''''''''''''''''''''''
Sub MinProcess
  Local Ax1m,RemainBC,AvBC,rg_mins,rg_abv,rg_asv,rg_aic,rg_alc,rg_abc,yrg
  ' Check for minute flag, calculate the average of the last 60 seconds of
  ' readings, plug them into the log file array at the log file array pointer
  ' calculate averages for last minute
  PreviousABV = AverageBV
  AverageBV = 0
  AverageSV = 0
  AverageCV = 0
  AverageSC = 0
  AverageCC = 0
  AverageACC = 0
  AverageDCC = 0
  AverageLC = 0
  For Ax1m = 0 To 29   'don't forget only sampled every 2 secs
    AverageBV = AverageBV + BVArray(Ax1m)
    AverageSV = AverageSV + SVArray(Ax1m)
    AverageCV = AverageCV + CVArray(Ax1m)
    AverageSC = AverageSC + SAArray(Ax1m)
    AverageCC = AverageCC + CAArray(Ax1m)
    AverageACC = AverageACC + ACAArray(Ax1m)
    AverageDCC = AverageDCC + DCAArray(Ax1m)
    AverageLC = AverageLC + LAArray(Ax1m)
  Next Ax1m
  AverageBV = AverageBV/30
  AverageSV = AverageSV/30
  AverageCV = AverageCV/30
  AverageSC = AverageSC/30
  AverageCC = AverageCC/30
  AverageACC = AverageACC/30
  AverageDCC = AverageDCC/30
  AverageLC = AverageLC/30
  AverageIC = AverageACC + AverageDCC  'sum X250 car/solar plus X25000 mains chargers
  AverageBC = AverageIC - AverageLC
  ' Calculate battery change as percentage and fill charge state graph
  ' now calculate charge and paint capacity graph
  ' 12.1V equates to 30% charge - note this is a rough calculation
  '  - it should be done under no load.
  ' *** work needed here ***
  local range = 8   ' shape of curve
  local offset = 11.4   ' offset tp position range of values
  local curve = 9   ' shape determined by discharge current
  ' ... some rule of thumb values to position values in the 
  ' correct range
  curve = 10-(AverageLC/2.1)
  BatCharge = (((AverageBV-offset)*range)^2)-curve

  If Simulate = 1 then
    print str$(Batcharge,3,0)
  endif
  BatCharge = Int(BatCharge)
  If BatCharge < 1 Then BatCharge = 1
  If BatCharge > 100 Then BatCharge = 100
  ' FillBatGraph(BatCharge)
  FillBatGraph  BatCharge
  ' Calculate time to 30% discharge of battery
  RemainBC = BatCharge - 30 ' rough convert from 30% - 100% to 0Ahrs to 70AHrs remaining charge
  If RemainBC < 1 Then
    RemainBC = 1
  EndIf
  AvBC = 10
  Time2Discharge = RemainBC/AvBC  'if we have 70 AHrs and we discharge at 10AHrs
  ' then we have 7 hours of usage till 30%
  Colour RGB(white)
  Text 400,132,"            "
  Text 400,132,Str$(Time2Discharge,3,1)+"hrs"
  
  ' Now paint out running graph
  ' - first work out the current minute hour combo in minutes
  rg_mins = (((Val(Mid$(Time$,1,2)) Mod 6)*60))+(Val(Mid$(Time$,4,2)))
  rg_mins = rg_mins + 41               ' offset to start of graph
  ' now scale all values to fit in graph
  rg_abv = 450 - Int((AverageBV-11) * 20)
  If rg_abv > 450 Then rg_abv = 450
  rg_asv = 450 - Int((AverageSV-11) * 20)
  If rg_asv > 450 Then rg_asv = 450
  If rg_asv < 230 Then rg_asv = 230
  rg_aic = 450 - Int(AverageIC * 10)
    rg_alc = 450 - Int(AverageLC * 10)
  If (Sgn(AverageBC) = +1) Then
    rg_abc = 450 - Int(AverageBC * 10)
  Else
    rg_abc = 450 + Int(AverageBC * 10)
  EndIf
  ' paint out pixels into graph for running values
  For yrg = 230 To 450 Step 1
    If rg_abc = yrg Then
      If (Sgn(AverageBC) = +1) Then
        Pixel rg_mins,yrg,RGB(green) ' average battery charge current in green
      Else
        Pixel rg_mins,yrg,RGB(red)  ' average battery discharge current in red
      EndIf
    Else
      Pixel rg_mins,yrg,RGB(black)  ' else just paint black pixel
    EndIf
    If rg_abv = yrg Then
      Pixel rg_mins,yrg,RGB(white)  ' average battery volts
    EndIf
    If rg_asv = yrg Then
      Pixel rg_mins,yrg,RGB(cyan)   ' average solar volts in cyan
    EndIf
    If rg_aic = yrg Then
      Pixel rg_mins,yrg,RGB(yellow) ' average input charging current in yellow
    EndIf
    If rg_alc = yrg Then
      Pixel rg_mins,yrg,RGB(red)  ' average load current in red
    EndIf
    Line(rg_mins+2),yrg,(rg_mins+3),yrg,1,RGB(black) 'mark current time
    Line(rg_mins+1),yrg,(rg_mins+2),yrg,1,RGB(white) 'mark current time
  Next yrg
  MinFlag = 0   ' reset the min flag
End Sub  ' End of minute code
''''''''''''''''''
' Six minute process - calculates the battery state of charge
' from averages of voltage and current draw over last 6 minutes
sub SixMinProcess
  SixMinFlag = 0  ' reset flag
end sub   ' End of 6 minute code
'''''''''''''''''

' End of Day process - save values for historical records,
' tidy up and reset. Note EOD currently 0600 Hrs.
Sub EODProcess  ' Check for end of day
  ' - for new day starting at 6AM
  RTC GETTIME        ' date and time from DS3231
  PrevIWH = InputWH
  InputWH = 0
  PrevLWH = LoadWH
  LoadWH = 0
  Day4BWH = Day3BWH
  Day3BWH = Day2BWH
  Day2BWH = PrevBWH
  TotalBWH = PrevBWH + Day2BWH + Day3BWH + Day4BWH
  PrevBWH = BatWH
  BatWH = 0
  TotalBWH = PrevBWH + Day2BWH + Day3BWH + Day4BWH
  EODFlag = 0    ' clear end of day flag until next 6AM
End Sub  ' End of EOD code
'''''''''''''''''
    
''''''''''''''''''''''''''
'  2 Second Interrupt Routine Handling to send data request
Sub RequestData      ' Timer Interrupt handler
  '   timer = 0
  ' Send data request to front end
  Print #1,Chr$(6);
  Print "Data request sent at ",time$ 'debug use
  DataValid = 0  ' reset ready for new data
  'flip twosecled on-off
  if twosecled = 1 then
    ctrlval(#11) = 1
    twosecled = 0
  else
    ctrlval(#11) = 0
    twosecled = 1
  endif
End Sub

'   Interrupt to receive data string from front end with all data values
Sub GetVals  ' Receive character interrupt handler
  Local BytesReady
  ' structure of DataVal$ -
  ' battery volts vv.vv, solar volts vv.v, car volts vv.vv,
  ' solar amps ii.i, car amps ii.i,
  ' ac charge amps ii.i, dc charge amps ii.i,
  ' load amps ii.i, data valid zz.z  in the form
  ' vv.vv,vv.v,vv.vv,ii.i,ii.i,ii.i,ii.i,ii.i,zz.z
  ' total 46 bytes  
  pause 50    ' wait for data in buffer to stabilise
  BytesReady = Loc(#1)
  Print BytesReady,    " characters waiting"
  DataVals$ = Input$(BytesReady,#1)
  if asc(left$(DataVals$,1)) = 6 then
    print "ASCII 6 loopback character received" ' debug use
    DataVals$ = "12.62,19.5,13.85,11.5,12.6,24.1,10.1,14.8,zz.z"
  endif
  Print DataVals$
  BVolts = Val(Mid$(DataVals$,1,5))
  SVolts = Val(Mid$(DataVals$,7,4))
  CVolts = Val(Mid$(DataVals$,12,5))
  SAmps  = Val(Mid$(DataVals$,18,4))
  CAmps = Val(Mid$(DataVals$,23,4))
  ACAmps = Val(Mid$(DataVals$,28,4))
  DCAmps= Val(Mid$(DataVals$,33,4))
  LAmps = Val(Mid$(DataVals$,38,4))
  If Mid$(DataVals$,43,4) = "zz.z" Then
  ' very simple checksum test
    print "Data valid" ' debug use
    DataValid = 1
  Else
    DataValid = 0
  EndIf
  IAmps = DCAmps + ACAmps
  BAmps = IAmps - LAmps
  ' grab time into component variables
  hrs = Val(left$(Time$,2))
  mins = Val(Mid$(Time$,4,2))  
  secs = Val(right$(Time$,2))
  ' Get the current time, extract the minutes component
  If secs = 0 Then
    MinFlag = 1
  Else
    MinFlag = 0
  EndIf
  ' Get the current time, test for 06:00
  If hrs = 6 And mins = 0 And secs = 0 Then
    EODFlag = 1
  Else
    EODFlag = 0
  EndIf
  NewDataFlag = 1 ' set to indicate fresh data
'  print "Exit get data interrupt at ",timer
End Sub ' Returns with the following values:-
  ' BVolts = 0 to 15 equalling battery volts
  ' SVolts = 0 to 25 equalling solar Array volts
  ' CVolts = 0 to 25 equalling solar Array volts
  ' SAmps = 0 to 50 equalling 0 to 50Amps
  ' CAmps = 0 to 50 equalling 0 to 50Amps
  ' ACAmps=0 to 50 equalling 0 to 50Amps
  ' DCAmps=0 to 50 equalling 0 to 50Amps
  ' LAmps=0 to 50 equalling 0 to 50Amps
  ' plus flags set/clear
'End of Interrupt code
'''''''''''''''''''''

' Defined Subroutines
'''''''''''''''''''''
' Set up terminal display with static data (field names etc.)
Sub DisplayStaticText
  Local xrg,yrg,volts,amps
  Text 180,1,"CARAVAN POWER MONITOR",LT,2,1,RGB(white)
  Text 432,6,Str$(F_Ver,2,2),LT,1,1,RGB(white)
  Line 180,22,470,22,2,RGB(white)
  Text 2,28,"Average Battery Voltage",LT,1,1,RGB(white)
  Text 200,28,"Min Average Readings",LT,1,1,RGB(white)
  Line 200,41,360,41,1,RGB(white)
  Text 200,54,"Battery Volts =",LT,1,1,RGB(white)
  Text 400,28,"Charge Log",LT,1,1,RGB(white)
  Text 400,41,"day   WHrs",LT,1,1,RGB(white)
  Line 400,53,480,53,1,RGB(white)
  Text 400,54,"-2",LT,1,1,RGB(white)
  Text 400,67,"-3",LT,1,1,RGB(white)
  Text 400,80,"-4",LT,1,1,RGB(white)
  Text 396,106,"Tot",LT,1,1,RGB(white)
  Text 200,67,"Solar Volts   =",LT,1,1,RGB(cyan)
  Text 200,80,"Vehicle Volts =",LT,1,1,RGB(180,180,180)
  Text 200,93,"Input Current =",LT,1,1,RGB(yellow)
  Text 200,106,"Load Current  =",LT,1,1,RGB(red)
  Text 200,119,"Bat Chg/",LT,1,1,RGB(green)
  Text 264,119,"Dischg=",LT,1,1,RGB(red)
  Text 2,145,"Daily Accumulating Vals",LT,1,1,RGB(white)
  Text 200,145,"Previous Days Total Vals",LT,1,1,RGB(white)
  Text 396,119,"Time to 30%",LT,1,1,RGB(white)
  Line 2,158,182,158,1,RGB(white)
  Line 200,158,390,158,1,RGB(white)
  Text 2,171,"Input Charge  =",LT,1,1,RGB(white)
  Text 200,171,"Input Charge   =",LT,1,1,RGB(white)
  Text 2,184,"Load Discharge=",LT,1,1,RGB(white)
  Text 200,184,"Load Discharge =",LT,1,1,RGB(white)
  Text 2,197,"Bat Chg/",LT,1,1,RGB(green)
  Text 66,197,"Dischg=",LT,1,1,RGB(red)
  Text 200,197,"Bat Chg/",LT,1,1,RGB(green)
  Text 264,197,"Dischg=",LT,1,1,RGB(red)
  Box 1,220,800,260,1,RGB(blue)
  Text 478,230,"Battery Volts",LT,1,1,RGB(white)
  Text 650,230,"Car Volts",LT,1,1,RGB(180,180,180)
  Text 478,270,"Solar Volts",LT,1,1,RGB(cyan)
  Text 650,270,"Solar Amps",LT,1,1,RGB(cyan)
  Text 478,310,"Input Amps",LT,1,1,RGB(yellow)
  Text 650,310,"Car Amps",LT,1,1,RGB(180,180,180)
  Text 478,350,"Load Amps",LT,1,1,RGB(red)
  Text 650,350,"AC Amps",LT,1,1,RGB(brown)
  Text 478,390,"Battery Amps",LT,1,1,RGB(white)
  Text 650,403,"DC Amps",LT,1,1,RGB(brown)
  Text 486,403,"Chg/",LT,1,1,RGB(green)
  Text 519,403,"Dischg",LT,1,1,RGB(red)
end sub

sub DisplayStaticRunGraph
' draw out static running graph frame, ticks etc.
  local volts,amps,yrg,xrg
  Colour RGB(white),RGB(black)  
  volts=22
  amps = 22
    For yrg =224 To 463 Step 20
    Text 6,yrg,Str$(volts,2,0)+"V",LT,1,1,RGB(white)
    Text 406,yrg,Str$(amps,2,0)+"A",LT,1,1,RGB(white)
    volts=volts-1
    amps=amps-2
  Next yrg
  Line 40,230,40,451
  Line 400,230,400,451
  For yrg = 230 To 450 Step 10 'minor ticks
    Pixel 39,yrg,RGB(white)
    Pixel 401,yrg,RGB(white)
  Next yrg
  For yrg = 230 To 450 Step 20  'major ticks
    Pixel 37,yrg,RGB(white)
    Pixel 38,yrg,RGB(white)
    Pixel 402,yrg,RGB(white)
    Pixel 403,yrg,RGB(white)
  Next yrg
  Line 40,451,400,451 'now for the bottom
  For xrg = 40 To 400 Step 15
    Line xrg,451,xrg,453,1,RGB(white)
  Next xrg
  For xrg = 40 To 400 Step 30
    Line xrg,451,xrg,454,1,RGB(white)
  Next xrg
  For xrg = 40 To 400 Step 60
    Line xrg,451,xrg,456,1,RGB(white)
  Next xrg
  Text 40,460,"6 Hour periods - 0600, then 1200, 1800, 2400",LT,1,1,RGB(white)
End Sub
''''''''
' Battery Graph Frame
Sub DisplayStaticBatGraph
  Local xtl,ytl,xbr,ybr,percent,xpos,ypos,y
  xtl=500:ytl=1
  '  First draw the box and batteryvalues
  Box xtl,ytl,300,220,1,RGB(blue)
  Box xtl+10,ytl+65,210,110,5,RGB(white)
  Box xtl+15,ytl+70,200,100,1,RGB(black),RGB(blacK)
  Box xtl+15,ytl+139,200,30,1,RGB(red),RGB(red)
  Box xtl+35,ytl+50,40,15,1,RGB(white),RGB(white)
  Box xtl+155,ytl+50,40,15,1,RGB(white),RGB(white)
  percent=100
  Text 560,4,"100%",LT,3,2,RGB(white)
End Sub
'''''''''

' Fill battery graph showing percentage charge
sub FillBatGraph (BatCharge)
  local bxtl,yline,batchg,ycntr
  bxtl = 517
  batchg = BatCharge
  for yline = 140 to 70 step -1
    ycntr = 170 - yline
    line bxtl,yline,bxtl+195,yline,,rgb(black)
    if batchg => ycntr  and ycntr > 30 then
      line bxtl,yline,bxtl+195,yline,,rgb(yellow)
    endif
    if batchg => ycntr and ycntr > 50 then
      line bxtl,yline,bxtl+195,yline,,rgb(green)
    endif
  next yline
  Text 560,3,"     ",LT,3,2,RGB(white)
  Text 560,3,str$(BatCharge,3,0)+"%",LT,3,2,RGB(white)
end sub
''''''''
Sub ChargeLog(PreviousBCAH,Day2BCAH,Day3BCAH,Day4BCAH,TotBCAH)
 Box 600,1,200,220,1,RGB(blue)
End Sub
'''''''
' If screen is touched in the graph area, 
' brightness is bumped by 25%
Sub PenDown
  If CtrlVal(#10) Then
    S_Bright = S_Bright + 25
    If S_Bright > 100 Then
      S_Bright = 0
    EndIf
    BackLight S_Bright
  EndIf
End Sub

' End program.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
