  '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  ' Battery Management Program for Caravan 12V System         '
  ' Copyright Doug Pankhurst July 2016                        '
  ' Written for MMBasic V5.4, copyright Geoff Graham 2012     '
  '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  ' File Version:    3.2 10 Aug 2017   cvbmfe32.bas
  ' - last known operational - cvbmfe242.bas
  ' Voltage and current sampling front end using PIC32MX170
  ' using Circuit Gizmos CG Microkit module.
  ' Additional electronics on veroboard backpack.
  ' Current sampling done by ACS758 modules.
  ' Values are sampled on receipt of back end query then sent via
  ' COM2 to back end for display.

  ' Global Variables - initialised to default values
  Dim BVolts = 12.8    '  Battery Voltage read from Pin 23 -  Read as
  ' 1/6th of real voltage via 6:1 resistive divider
  Dim BV$
  Dim SVolts = 20      ' Solar Array voltage from P in 22. Read as 1/10th
  ' of real voltage by 10:1 resistive divider
  Dim SV$
  Dim PVACurrent = 10  ' Charging current fromPVA into CTEK D250S
  Dim PVAC$
  Dim CVolts = 13.8    ' Car battery voltage from Pin 22. Read as 1/10th
  ' of real voltage by 6:1 resistive divider
  Dim CV$
  Dim CarCurrent = 10  ' Charging current from vehicle into CTEK D250S
  Dim CC$
  Dim DCCurrent = 12  ' Charging current into battery/load from CTEK D250S - solar and car
  Dim DC$
  Dim ACCurrent = 12  ' Charging current into battery/load from CTEK XS25000 240VAC charger
  Dim AC$
  Dim LoadCurrent = 16 ' Load current of caravan - can be supplied from the
  ' batteries and the charging systems
  Dim LC$
  Dim BatCurrent = 6   ' A calculated value; if positive, = a charging current to
  ' the battery; if negative, equals a discharge current from the battery.
  Dim BC$
  Dim ic    ' loop counter
  Dim ADCOffset = 0.062  ' a to d offset for zero
  Dim BVAdj = 0.815  ' adjustment to BV for correct reading
  Dim SVAdj = 0.784  ' adjustment to SV for correct reading
  Dim CVAdj = 0.818  ' adjustment to CV for correct reading
  Dim DataVals$      ' concatinated string to send to BE
  Dim DataReq = 0        ' data request flag
  Dim getpin23,getpin24,getpin25,getpin2,getpin3,getpin4,getpin6
  Dim NewDataFlag = 0    ' Two second flag - set every 2 seconds by the

  '''''''''''''''''
  ' Carry out initialisation routines here - enable interrupt last
  ' Main Entry to Program
  OPTION EXPLICIT

  ' Set up all initial values to default for orderly entry.
  ' configure uMite 170  pins
  SetPin 2,AIN  'J1-6 - solar current - 0 to 2.5V = 0 to 50A
  SetPin 3,AIN  'J1-5 - car current - 0 to 2.5V = 0 to 50A
  SetPin 4,AIN   'J1-8 -AC charging current - 0 to 2.5V = 0 to 50A
  SetPin 6,AIN   'J1-7 -DC charging current - 0 to 2.5V = 0 to 50A
  SetPin 5,AIN   'J1-9 - load current - 0 to 2.5V = 0 to 50A
  SetPin 26,AIN  '  - battery volts - 0 to 2.5V = 0 to 15V
  SetPin 23,AIN  '  - solar volts - 0 to 2.5V = 0 to 25V
  SetPin 24,ain  '  - car battery volts - 0 to 2.5 = 0 to 15V
  SetPin 15,dout '  - pulse on/off each sample period

  Open "COM2:9600,128,GetPinData" As #1   ' uMite Tx on pin 8, 
  ' uMite Rx on pin 9, generate interrupt when character received

  ' end of initialisation routines
  ''''''''''''''''''''
  ' Main program loop

Main:
  Do
    Do While DataReq = 0  ' wait until back end data request to send fresh data
    ' - this should occur every 2 seconds
    Loop
      ic = 0
      BVolts = 0
      GetPin26= 0
      For ic = 1 To 10
        GetPin26 = Pin(26)
        BVolts = BVolts + GetPin26
      Next ic
      BVolts = BVolts/10  ' Average out Battery voltage
      BVolts = (BVolts*6)*BVAdj   ' 15 battery volts = 2.5 MM volts
      BVolts = FixDecPlace(BVolts,2)
      
      ' Get the Solar Array voltage
      ic = 0
      SVolts=0
      GetPin23= 0
      For ic = 1 To 10
        GetPin23 = Pin(23)
        SVolts = SVolts + GetPin23
      Next ic
      SVolts = SVolts/10  ' same as for BVolts
      SVolts = (SVolts*10)*SVAdj  ' 25 solar volts = 2.5 MM volts
      SVolts = FixDecPlace(SVolts,1)
      
      ' Get the vehicle battery voltage
      ic = 0
      CVolts=0
      GetPin24= 0
      For ic = 1 To 10
        GetPin24 = Pin(24)
        CVolts = CVolts + GetPin24
      Next ic
      CVolts = CVolts/10  ' same as for BVolts
      CVolts = (CVolts*6)*CVAdj  ' 15 car volts = 2.5 MM volts
      CVolts = FixDecPlace(CVolts,2)
      
      ' Get the solar panel current as a voltage
      ic = 0
      PVACurrent = 0
      GetPin2 = 0
      For ic = 1 To 10
        GetPin2 = Pin(2) - 2.5 ' ACS758 module
        ' 2.5V is 0A, 5V is 50A
        PVACurrent = PVACurrent + GetPin2
      Next ic
      PVACurrent = Abs(PVACurrent/10) ' Average out input current
      PVACurrent = PVACurrent*25 ' current by 25 gives 40mV=1A
      PVACurrent = FixDecPlace(PVACurrent,1)

      ' Get the anderson plug current as a voltage
      ic = 0
      CarCurrent = 0
      GetPin3 = 0
      For ic = 1 To 10
        GetPin3 = Pin(3) - 2.5
        ' 2.5V is 0A, 5V is 50A
        CarCurrent = CarCurrent + GetPin3
      Next ic
      CarCurrent = Abs(CarCurrent/10)   ' Average out input current
      CarCurrent = CarCurrent*25   ' current by 25 gives 40mV=1A
      CarCurrent = FixDecPlace(CarCurrent,1)

      ' Get the D250S charging current as a voltage
      ic = 0
      DCCurrent = 0
      GetPin6 = 0
      For ic = 1 To 10
        GetPin6 = Pin(6) - 2.5
        ' 2.5V is 0A, 5V is 50A
        DCCurrent = DCCurrent + GetPin6
      Next ic
      DCCurrent = Abs(DCCurrent/10) ' Average out load current
      DCCurrent = DCCurrent*25   ' current by 25 gives 40mV=1A
      DCCurrent = FixDecPlace(DCCurrent,1)

      ' Get the XC25000 Mains charging current as a voltage
      ic = 0
      ACCurrent = 0
      GetPin4 = 0
      For ic = 1 To 10
        GetPin4 = Pin(4) - 2.5
        ' 2.5V is 0A, 5V is 50A
        ACCurrent = ACCurrent + GetPin4
      Next ic
      ACCurrent = Abs(ACCurrent/10) ' Average out load current
      ACCurrent = ACCurrent*25   ' current by 25 gives 40mV=1A
      ACCurrent = FixDecPlace(ACCurrent,1)
      
      ' Get the load current as a voltage
      ic = 0
      LoadCurrent = 0
      GetPin5 = 0
      For ic = 1 To 10
        GetPin5 = Pin(5) - 2.5
        ' 2.5V is 0A, 5V is 50A
        LoadCurrent = LoadCurrent + GetPin5
      Next ic
      LoadCurrent = Abs(LoadCurrent/10) ' Average out load current
      LoadCurrent = LoadCurrent*25   ' current by 25 gives 40mV=1A
      LoadCurrent = FixDecPlace(LoadCurrent,1)

    ' Format all values to strings
    BV$ = Str$(BVolts,2,2)
    SV$ = Str$(SVolts,2,1)
    CV$ = Str$(CVolts,2,2)
    PVAC$ = Str$(PVACurrent,2,1)
    CC$ = Str$(CarCurrent,2,1)
    DCC$ = Str$(DCCurrent,2,1)
    ACC$ = Str$(ACCurrent,2,1)
    LC$ = Str$(LoadCurrent,2,1)
    DataVals$ = BV$+","+SV$+","+CV$+","+PVAC$+","+CC$+","
    DataVals$ = DataVals$+DCC$+","+ACC$+","+LC$+","+"zz.z"

    ' Returns with the following values:-
    ' BVolts = 0 to 15 - van battery volts
    ' SVolts = 0 to 25 - solar array volts
    ' CVolts = 0 to 15 - vehicle battery volts
    ' PVACurrent = 0 to 50 - solar panel current
    ' CarCurrent = 0 to 50 - vehicle current
    ' DCCurrent = 0 to 50 - combined solar and vehicle charging current
    ' ACCurrent = 0 to 50 - mains charging current
    ' LoadCurrent = 0 to 50 - van load current

    Print #1,DataVals$;    ' dump data out COM2
    Print DataVals$   '  dump out local port for testing/monitor
    DataReq = 0  ' reset DataReq flag
    ' Heartbeat LED pulses every 2 sec
    If Pin(15) = 1 Then
      Pin(15)=0
    Else
      Pin(15)=1
    EndIf
  Loop
  ' End of main program loop
  ''''''''''''''''''''''''''
  ' Interrupt Routine Handling

  ' Interrupt by COM2 receive data, check for ACK else ignore
  Sub GetPinData
  local ACK$ length 5
    ACK$ = Input$(1,#1)
    Print ACK$+"  Character received" ' for debug
    If Asc(ACK$) = 6 Then
      DataReq = 1
    Else
      DataReq = 0
    EndIf
  End Sub

  'End of Interrupt code
  '''''''''''''''''''''
  ' Defined Subroutines
  '''''''''''''''''''''
  Function FixDecPlace(num,decplaces)
    Local n,dp
    n = num
    dp = decplaces
    n = n * (10 ^ dp)
    n = Cint(n)
    n = n/(10^dp)
    FixDecPlace = n
  End Function

  ' End program.
  ''''''''''''''''''''''''''''''''''''''''''''''
